"""Rayvision logging."""

# Disabling the import error because PyLint chokes on importing pkg_resources,
# it is available though.
# pylint: disable=import-error
from pkg_resources import DistributionNotFound
from pkg_resources import get_distribution

from rayvision_log.core import init_logger
from rayvision_log.core import set_up_logger

__all__ = ["init_logger", "set_up_logger"]

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    # Package is not installed.
    __version__ = "0.0.0-dev.1"



"""Test rayvision_log core functions."""
import logging

from rayvision_log.core import init_logger, set_up_logger


def test_overwrite(monkeypatch, tmpdir):
    """Test we can overwrite the log output root."""
    root = tmpdir.join("test_log")
    monkeypatch.setenv("RAYVISION_LOG_ROOT", str(root))
    init_logger("rayvision_log_test")
    logger = logging.getLogger("rayvision_log_test")
    logger.info("test")
    assert len(root.listdir()) == 1


def test_init_logger():
    """Test init the custom logger."""
    config = {
        "version": 1,
        "formatters": {
            "file": {
                "format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"
            }  # noqa: E501 # pylint: disable=line-too-long
        },
        "handlers": {
            "file": {
                "class": "concurrent_log_handler.ConcurrentRotatingFileHandler",  # noqa: E501 # pylint: disable=line-too-long
                "formatter": "file",
                "level": "DEBUG",
                "filename": "",
                "backupCount": 7,
                "maxBytes": "1024*1024*5",
                "delay": True,
            }
        },
        "loggers": {"": {"level": "DEBUG", "handlers": ["file"]}},
    }
    set_up_logger("rayvision_log_test", config)
    logging.basicConfig()
    logger = logging.getLogger("rayvision_log_test")
    logger.info("test")
    assert "rayvision_log_test" in logging.root.manager.loggerDict


def test_catch_setup_logger_failed(mocker):
    """Test that we can ignore the initialization log error.

    The idea behind this is that if we fail to initialize the log we will not
    affect the use of the log API and will be able to catch the wrong log.

    """
    mock_config = mocker.patch("rayvision_log.core.get_default_log_config")
    mock_config.return_value = {}
    init_logger("test_logger")
    assert "rayvision_log" in logging.root.manager.loggerDict



"""Describe our module distribution to Distutils."""
from setuptools import find_packages, setup


def parse_requirements(filename):
    with open(filename, "r") as f:
        for line in f:
            yield line.strip()


setup(
    name="rayvision_log",
    author="Shenzhen Rayvision Technology Co., Ltd",
    author_email="developer@rayvision.com",
    url="https://github.com/renderbus/rayvision_log",
    package_dir={"": "."},
    packages=find_packages("."),
    description="A Python-based API for Using Renderbus cloud rendering service.",
    entry_points={},
    install_requires=list(parse_requirements("requirements.txt")),
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    package_data={"rayvision_log": ["*.yaml"]},
)



"""Basic logging setup."""
from copy import deepcopy
from getpass import getuser
import logging.config
import os
from socket import getfqdn
import traceback

from appdirs import user_log_dir
import yaml


def init_logger(app_name):
    """Initialize logging with our default configuration.

    An easier interface for artists to use to create loggers in their
    packages and applications. Any application that uses this will use the
    default logging configuration.

    Examples:
        >>> import logging
        >>> from rayvision_log import init_logger
        >>> app_name = "rayvision_api"
        >>> init_logger(app_name)
        >>> LOG = logging.getLogger(app_name)

    Args:
        app_name (str): The application or package name for which to create a logger.
    """
    try:
        log_config = get_default_log_config()
        set_up_logger(app_name, log_config)
    except ValueError:
        set_up_logger(app_name, log_config, use_orginal=True)

    except Exception:  # pylint: disable=broad-except
        # Catch a broad exception here, since there's no reason logging setup
        # failing should crash a program.
        err_msg = traceback.format_exc()
        logging.basicConfig()
        logger = logging.getLogger("rayvision_log")
        logger.exception("rayvision log initilization failed.\n%s", err_msg)


def get_default_log_config():
    """Get the default logging configuration.

    Returns:
        dict: The default logging configuration.

    """
    root = os.path.dirname(__file__)
    config_file = os.path.join(root, "logging.yaml")
    with open(config_file, "r") as file_object:
        data = yaml.load(file_object, yaml.FullLoader)
    return data["logging"]

def update_config(config):
    """Some python2 versions use native logging

    Args:
        config (dict): logging config

    Returns:
        dict: logging config.

    """
    config['handlers']['file']['class'] = 'logging.handlers.RotatingFileHandler'
    return config

def set_up_logger(app_name, log_config, use_orginal=False):
    """Set up loggers based on given name and configuration.

    Example:
        >>> app_name = "rayvision_api"
        >>> log_config = {
                            formatters: {
                                file: {
                                    format: '%(asctime)s [%(levelname)s] %(name)s: %(message)s'
                                }
                            },
                            handlers: {
                                file: {
                                    class: concurrent_log_handler.ConcurrentRotatingFileHandler
                                }
                            }
                        }

    Args:
        app_name (str): Name of the application.
        log_config (dict): The configuration of the logger.

    References:
        https://pypi.org/project/ConcurrentLogHandler/

    """
    log_config = deepcopy(log_config)
    if use_orginal:
        log_config = update_config(log_config)
    if log_config["handlers"].get("file"):
        root = os.getenv(
            "RAYVISION_LOG_ROOT", user_log_dir(app_name, appauthor="RayVision")
        )
        filename = os.path.join(root, getuser(), "{}.log".format(getfqdn()))
        log_config["handlers"]["file"]["filename"] = filename
        folder = os.path.dirname(filename)
        if not os.path.isdir(folder):
            os.makedirs(folder)
    logging.config.dictConfig(log_config)
